@@ -247,6 +247,24 @@
 
 (function( $, window, undefined ) {
 
+    function makeQuery(data) {
+        var query = {};
+        $.each(data, function(i, field) {
+            if (query[field.name] === undefined) {
+                query[field.name] = [];
+            }
+            query[field.name].push(field.value);
+        });
+        return query;
+    }
+
+    function deserialize($form, data) {
+        $.each(data, function() {
+            console.log("deserialize", this.name, this.value);
+            $form.find("[name='" + this.name + "']").val(this.value);
+        });
+    }
+
     function defaultRefClickCallback() {
         var $item = $(this),
             $library = $item.closest(":jqmData(role='library')"),
@@ -375,13 +393,15 @@
             .mopidyModel(track);
     }
 
-    $.widget( "mopidy.library", $.mobile.listview, {
+    $.widget("mopidy.library", $.mobile.listview, {
 
 	initSelector: ":jqmData(role='library')",
 
         options: {
             back: null,
             forward: null,
+            search: null,
+            searchExact: false,
             title: null  // label, caption?
         },
 
@@ -413,20 +433,6 @@
                     }
                     event.preventDefault();
                 },
-                "filterablebeforefilter": function(event, data) {
-                    var $input = $(data.input),
-                        query = $input.val(),
-                        uri = widget.getCurrentUri();
-                    $.mopidy.connect().then(function() {
-                        if (query && query.length > 0) {
-                            widget._search(this, {any: [query]}, uri ? [uri] : null);
-                        } else {
-                            widget._browse(this, uri);
-                        }
-                    });
-                    event.preventDefault();
-                    event.stopPropagation();
-                }
             });
 
             // FIXME: _setOption, _init?
@@ -436,9 +442,19 @@
                 });
             });
 
-            // work around default filter
-            $element.filterable("option", "filterCallback", function() {
-                return false;
+            $(options.search).submit(function(event) {
+                var $form = $(event.target),
+                    data = $form.serializeArray(),
+                    exact = widget.options.searchExact,
+                    title = $(widget.options.title).text(),
+                    uri = widget.getCurrentUri();
+                console.log("search: ", event, data);
+                $.mopidy.connect().then(function() {
+                    widget._search(this, makeQuery(data), uri ? [uri] : null, exact);
+                    // FIXME: special title for search?
+                    widget._pushState({data: data, exact: exact}, title, uri);
+                });
+                event.preventDefault();
             });
 
             this._pushState(null, $(options.title).text(), null);
@@ -523,13 +539,19 @@
                 widget = this;
             if (index >= 0 && index < history.length) {
                 $.mopidy.connect().then(function() {
-                    widget._browse(this, history[index].uri);
+                    var state = history[index].state,
+                        uri = history[index].uri;
+                    if (state) {
+                        widget._search(this, makeQuery(state.data), uri ? [uri] : null, state.exact);
+                        deserialize($(widget.options.search), state.data);
+                    } else {
+                        widget._browse(this, uri);
+                        $(widget.options.search).trigger("reset");
+                    }
                 });
                 $(options.back).toggle(index > 0);
                 $(options.forward).toggle(index < history.length - 1);
                 $(options.title).text(history[index].title || "");
-                // data-input not in options?
-                $(widget.element.jqmData("input")).val("");
                 this._currentHistoryIndex = index;
             }
         },
@@ -904,12 +926,12 @@
             });
         },
 
-        refresh: function(/*uriScheme*/) {
-            //if (uriScheme !== undefined) {
-            //    $.mopidy.connect().then(function() {
-            //        this.playlists.refresh(uriScheme);
-            //    });
-            //}
+        refresh: function(uriScheme) {
+            if (uriScheme !== undefined) {
+                $.mopidy.connect().then(function() {
+                    this.playlists.refresh(uriScheme);
+                });
+            }
             this._super();
         },
 
